/*
 * uart.c - serial hw routines
 *
 * Written by
 *  Christian Vogelgsang <chris@vogelgsang.org>
 *
 * This file is part of dtv2ser.
 * See README for copyright notice.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

#include "global.h"

#include <avr/io.h>
#include <avr/interrupt.h>

#include "uart.h"
#include "timer.h"
#include "util.h"
#include "param.h"
#include "display.h"

#ifdef UBRR0H

// for atmeg644
#define UBRRH  UBRR0H
#define UBRRL  UBRR0L
#define UCSRA  UCSR0A
#define UCSRB  UCSR0B
#define UCSRC  UCSR0C
#define UDRE   UDRE0
#define UDR    UDR0

#define RXC    RXC0
#define TXC    TXC0
#define DOR    DOR0
#define PE     UPE0

#endif

// calc ubbr from baud rate
#define UART_UBRR   F_CPU/16/UART_BAUD-1

void uart_init(void) {
  // baud rate
  UBRRH = (u08)((UART_UBRR)>>8);
  UBRRL = (u08)((UART_UBRR)&0xff);

  UCSRB = 0x18; // 0x18  enable tranceiver and transmitter, no interrupt
  UCSRC = 0x86; // 0x86 -> use UCSRC, 8 bit, 1 stop, no parity, asynch. mode
}

u08 uart_data_available(void) {
  // UCSRA Bit 7  RXC: USART Receive Complete
  if (((UCSRA >> RXC) & 1) == 0) {
    return 0;
  }
  return 1;
}

#if 0
u08 uart_data_transmitted(void) {
  // Bit 6  TXC: USART Transmit Complete
  if (((UCSRA >> TXC) & 1) == 0) {
    return 0;
  }
  return 1;
}

u08 uart_transmit_buffer_ready(void) {
  // Bit 5  UDRE: USART Data Register Empty
  if (((UCSRA >> UDRE) & 1) == 0) {
    return 0;
  }
  return 1;
}

u08 uart_data_overrun(void) {
  // Bit 3  DOR: Data OverRun		-> specs state 'Always set this bit to zero when writing to UCSRA.'
  if (((UDRE >> DOR) & 1) == 0) {
    return 0;
  }
  return 1;
}

u08 uart_parity_error(void) {
  // Bit 2  PE: Parity Error			-> specs state 'Always set this bit to zero when writing to UCSRA.'
  if (((UDRE >> PE) & 1) == 0) {
    return 0;
  }
  return 1;
}
#endif

void uart_clear_buffer(void) {
  for (u08 i=0; i<2; i++) {
    while (uart_data_available() == 1) {
      u08 dummy;
      uart_read(&dummy);
    }
  }
}

// read/write without RTS/CTS

u08 uart_read(u08 *data) {
  // wait for RXC-Bit in UCSRA
  timer_100us = 0;
  u16 timeout = PARAM_WORD(PARAM_WORD_SERIAL_READ_AVAIL_TIMEOUT);
  while(((UCSRA >> RXC) & 1)==0) {
    if (timer_100us > timeout) {
      return 0;
    }
  }
  *data = UDR;
  return 1;
}

u08 uart_send(u08 data)
{
  // wait for transmitter to become ready
  timer_100us = 0;
  u16 timeout = PARAM_WORD(PARAM_WORD_SERIAL_SEND_READY_TIMEOUT);
  while(!( UCSRA & (1<<UDRE))) {
    if (timer_100us > timeout) {
      return 0;
    }
  }
  uart_send_direct(data);
  return 1;
}

// read/write with RTS/CTS

u08 uart_read_with_cts(u08 *data)
{
  uart_set_cts(1);
  u08 status = uart_read(data);
  uart_set_cts(0);
  return status;
}

u08 uart_send_with_rts(u08 data)
{
  // wait for rts with timeout
  timer_100us = 0;
  u16 rts_timeout = PARAM_WORD(PARAM_WORD_SERIAL_RTS_TIMEOUT);

#ifdef SHOW_WAIT_RTS
  led_ready_on();
#endif
  while(uart_get_rts()==0) {
    if(timer_100us > rts_timeout) {
#ifdef SHOW_WAIT_RTS      
      led_ready_off();
#endif
      return 0;
    }
  }
#ifdef SHOW_WAIT_RTS
  led_ready_off();
#endif
  return uart_send(data);
}

// ----- Tools -----

u08 uart_send_string(u08 *str)
{
  while(*str) {
    if(!uart_send(*str))
      return 0;
    str++;
  }
  return 1;
}

u08 uart_send_data(u08 *data,u08 len)
{
  for(u08 i=0;i<len;i++) {
    if(!uart_send(data[i]))
      return 0;
  }
  return 1;
}

u08 uart_send_crlf(void)
{
  return uart_send_string((u08 *)"\r\n");
}

static u08 buf[6];

u08 uart_send_hex_byte_crlf(u08 data)
{
  byte_to_hex(data,buf);
  if(uart_send_data(buf,2))
    return uart_send_crlf();
  else
    return 0;
}

u08 uart_send_hex_word_crlf(u16 data)
{
  word_to_hex(data,buf);
  if(uart_send_data(buf,4))
    return uart_send_crlf();
  else
    return 0;
}

u08 uart_send_hex_dword6_crlf(u32 data)
{
  dword_to_hex6(data,buf);
  if(uart_send_data(buf,6))
    return uart_send_crlf();
  else
    return 0;
}

